package com.sp.blinddate.util;

import com.sp.blinddate.base.LoginType;
import com.sp.blinddate.base.PModule;
import com.sp.blinddate.security.vo.SPSecurityUser;
import com.sp.blinddate.module.permission.vo.ModulePermissionVO;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * @author: jahe.lai
 * @create: 2019-09-24 10:31
 */
@Component
public class JwtUtil {

    /**
     * 用户名称
     */
    private static final String USERNAME = Claims.SUBJECT;

    private static final String USERID = "userid";
    /**
     * 创建时间
     */
    private static final String CREATED = "created";
    /**
     * 模块的权限列表 jahe.lai
     */
//    private static final String PERMISSION_LIST = "permissionList";
    /**
     * 登录类型，有可能是指登录的子系统类型 jahe.lai
     */
    private static  final String LOGINTYPE = "loginType";
    /**
     * 密钥
     */
    private static final String SECRET = "abcdefgh66666777778888888999999qwertyuioplkjhgfdsazxcvbnm9999";
    /**
     * 有效期 4小时
     */
    private static final long EXPIRE_TIME = 4 * 60 * 60 * 1000;

    @Value("${jwt.header}")
    private String tokenHeader;

    @Value("${jwt.tokenHead}")
    private String authTokenStart;

    /**
     * 生成令牌
     *
     * @return 令牌
     */
    public static String generateToken(SPSecurityUser userDetail) {
        Map<String, Object> claims = new HashMap<>(3);
        claims.put(USERID,userDetail.getUserId());
        claims.put(USERNAME, userDetail.getUsername());
        claims.put(CREATED, new Date());
//        claims.put(PERMISSION_LIST, userDetail.getPermissionList());
        claims.put(LOGINTYPE, userDetail.getLoginType());
        return generateToken(claims);
    }

    /**
     * 从数据声明生成令牌
     *
     * @param claims 数据声明
     * @return 令牌
     */
    private static String generateToken(Map<String, Object> claims) {
        Date expirationDate = new Date(System.currentTimeMillis() + EXPIRE_TIME);
        String token = Jwts.builder().setClaims(claims).setExpiration(expirationDate)
                .signWith(SignatureAlgorithm.HS256, SECRET).compact();
        return token;
    }

    /**
     * 从令牌中获取用户名
     *
     * @param token 令牌
     * @return 用户名
     */
    public static String getUsernameFromToken(String token) {
        Claims claims = getClaimsFromToken(token);
        return claims.getSubject();
    }

    /**
     * 根据请求令牌获取登录认证信息
     *
     * @return 用户名
     */
    public SPSecurityUser getUserFromToken(HttpServletRequest request) {
        // 获取请求携带的令牌
        String token = getToken(request);
        if (StringUtils.isNotEmpty(token)) {
            Claims claims = getClaimsFromToken(token);
            if (claims == null) {
                return null;
            }
            String username = claims.getSubject();
            if (username == null) {
                return null;
            }
            if (isTokenExpired(token)) {
                return null;
            }
            if (validateToken(token, username)){
                // 解析对应的权限以及用户id
                Integer userId = (Integer)claims.get(USERID);
                // 未把密码放到jwt
                LoginType loginType = LoginType.codeOf(claims.get(LOGINTYPE).toString());
                SPSecurityUser user = new SPSecurityUser(userId, username,"", loginType);
//                user.setPermissionList(getPermission(claims.get(PERMISSION_LIST)));
                return user;
            }
        }
        return null;
    }


    /**
     * jwt解析出权限信息，结果是一个List<LinkedHashMap<String, Object>.
     * key=pModule值的类型是字符串
     * key = nodeNameList的值是ArrayList
     * @param permissions
     * @return
     */
    private static List<ModulePermissionVO> getPermission(Object permissions) {
        List<ModulePermissionVO> permissionVOList = new ArrayList<>();
        PModule module;
        List<String> nodeNameList;
        if (permissions instanceof List) {
            for (Object object : (List) permissions) {
                Map<String, Object> map = (Map<String, Object>) object;
                module = PModule.valueOf(map.get("pModule").toString());
                nodeNameList = (List<String>) map.get("nodeNameList");
                permissionVOList.add(ModulePermissionVO.ofJWT(module, nodeNameList));
            }
        }
        return permissionVOList;
    }

    /**
     * 从令牌中获取数据声明
     *
     * @param token 令牌
     * @return 数据声明
     */
    private static Claims getClaimsFromToken(String token) {
        Claims claims;
        try {
            claims = Jwts.parser().setSigningKey(SECRET).parseClaimsJws(token).getBody();
        } catch (Exception e) {
            claims = null;
        }
        return claims;
    }

    /**
     * 验证令牌
     *
     * @param token
     * @param username
     * @return
     */
    private static Boolean validateToken(String token, String username) {
        String userName = getUsernameFromToken(token);
        return (userName.equals(username) && !isTokenExpired(token));
    }

    /**
     * 刷新令牌
     *
     * @param token
     * @return
     */
    public static String refreshToken(String token) {
        String refreshedToken;
        try {
            Claims claims = getClaimsFromToken(token);
            claims.put(CREATED, new Date());
            refreshedToken = generateToken(claims);
        } catch (Exception e) {
            refreshedToken = null;
        }
        return refreshedToken;
    }

    /**
     * 判断令牌是否过期
     *
     * @param token 令牌
     * @return 是否过期
     */
    private static Boolean isTokenExpired(String token) {
        try {
            Claims claims = getClaimsFromToken(token);
            Date expiration = claims.getExpiration();
            return expiration.before(new Date());
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 获取请求token
     *
     * @param request
     * @return
     */
    private String getToken(HttpServletRequest request) {
        String token = request.getHeader(tokenHeader);
        if (StringUtils.isNotEmpty(token)) {
            token = token.substring(authTokenStart.length());
        }
        return token;
    }
}
