package com.sp.blinddate.web.config;


import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.support.http.StatViewServlet;
import com.alibaba.druid.support.http.WebStatFilter;
import com.alibaba.druid.support.spring.stat.DruidStatInterceptor;
import org.springframework.aop.support.DefaultPointcutAdvisor;
import org.springframework.aop.support.JdkRegexpMethodPointcut;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.boot.web.servlet.ServletRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import javax.sql.DataSource;
import java.util.HashMap;
import java.util.Map;

/**
 * 配置durid数据库连接池的属性
 * 配置管理后台访问
 * 配置监控的链接
 * 配置对spring方法的监控, 配置见classpath:spring-bean.xml
 * @author jahe.lai
 */
@Configuration
public class DuridConfig {

    @ConfigurationProperties(prefix = "spring.datasource")
    @Bean
    public DataSource druid() {
        return new DruidDataSource();
    }

    @Value("${druid.monitor.loginUsername}")
    private String loginUserName;

    @Value("${druid.monitor.loginPassword}")
    private String loginPassword;

    /**
     * 配置Durid管理后台的访问链接
    **/
    @Bean
    public ServletRegistrationBean statViewServlet() {
        // 处理 druid下的所有请求
        ServletRegistrationBean bean = new ServletRegistrationBean(new StatViewServlet(), "/druid/*");
        Map<String, Object> initParams = new HashMap<>();
        initParams.put("loginUsername", loginUserName);
        initParams.put("loginPassword", loginPassword);
        initParams.put("deny", "");
        initParams.put("allow", "");
        bean.setInitParameters(initParams);
        return bean;
    }

    /**
     * 配置WebURI的监控filter
    **/
    @Bean
    public FilterRegistrationBean webStatFilter() {
        FilterRegistrationBean bean = new FilterRegistrationBean();
        bean.setFilter(new WebStatFilter());
        Map<String, Object> initParams = new HashMap<>();
        // 忽略 因为ContextPath设置有值，所以需要 */durid/*
        initParams.put("exclusions", " /durid/*, *.jpg,*.bmp,*.png, *.ico, *.js, *.css, *.html, *.json");

        bean.setInitParameters(initParams);
        // 拦截的请求
        bean.addUrlPatterns("/*");
        return bean;
    }

    // 配置对spring方法的监控，可只对dao层的方法监控，也可以指定多个代码层。
    // 需要导入sprign-aop的依赖支持，配置见classpath:spring-bean.xml
    // 启动类导入spring-bean配置文件 @ImportResource("classpath:spring-bean.xml")

    @Bean
    public DruidStatInterceptor druidStatInterceptor(){
        return new DruidStatInterceptor();
    }

    /**
     * 定义切点
    **/
    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public JdkRegexpMethodPointcut jdkRegexpMethodPointcut(){
        JdkRegexpMethodPointcut pointcut = new JdkRegexpMethodPointcut();
        pointcut.setPatterns("com.sp.friendship.admin.module.*.mapper.*");
        return pointcut;
    }

    @Bean
    public DefaultPointcutAdvisor druidStatAdvisor(DruidStatInterceptor druidStatInterceptor,
                                                   JdkRegexpMethodPointcut jdkRegexpMethodPointcut) {
        DefaultPointcutAdvisor defaultPointAdvisor = new DefaultPointcutAdvisor();
        defaultPointAdvisor.setPointcut(jdkRegexpMethodPointcut);
        defaultPointAdvisor.setAdvice(druidStatInterceptor);
        return defaultPointAdvisor;
    }
}
