package com.sp.blinddate.web.config;

import com.sp.blinddate.base.AUri;
import com.sp.blinddate.security.SPUserDetailsService;
import com.sp.blinddate.security.filter.ImageCodeFilter;
import com.sp.blinddate.security.filter.JwtAuthenticationTokenFilter;
import com.sp.blinddate.security.filter.MySecurityPrivilegeFilter;
import com.sp.blinddate.security.handler.SPAuthenticationEntryPointImpl;
import com.sp.blinddate.security.handler.SPAuthenticationFailureHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.web.access.intercept.FilterSecurityInterceptor;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

/**
 * @author: jahe.lai
 * @create: 2019-09-23 17:02
 */
@Configuration
@EnableWebSecurity
public class WebSecurityConfig extends WebSecurityConfigurerAdapter {

    @Autowired
    private SPAuthenticationEntryPointImpl authenticationEntryPointImpl;
    @Autowired
    private MySecurityPrivilegeFilter mySecurityPrivilegeFilter;

    @Autowired
    private SPAuthenticationFailureHandler authenticationFailureHandler;

    @Autowired
    private JwtAuthenticationTokenFilter jwtAuthenticationTokenFilter;

    @Autowired
    private ImageCodeFilter imageCodeFilter;

    @Autowired
    private SPUserDetailsService userDetailsService;

    /**
     * 解决 无法直接注入 AuthenticationManager
     *
     * @return
     * @throws Exception
     */
    @Bean
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }


    /**
     * 配置策略
     *
     * @param httpSecurity
     * @throws Exception
     */
    @Override
    protected void configure(HttpSecurity httpSecurity) throws Exception {
        httpSecurity
                // 由于使用的是JWT，我们这里不需要csrf
                .csrf().disable()
                //.anonymous().disable()
                .cors().and()
                // 认证失败处理类
                .exceptionHandling().authenticationEntryPoint(authenticationEntryPointImpl).and()
                // 基于token，所以不需要session
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS).and()
                // 过滤请求
                .authorizeRequests()
                // 对于登录login 图标 要允许匿名访问
                .antMatchers(HttpMethod.GET,
                        "/webjars/**","/v2/api-docs","/favicon.ico", "/*.html","/**/*.html","/**/*.css","/**/*.js")
                .permitAll()
                .antMatchers( AUri.LOGIN,
                          AUri.LOGOUT,
                          "/webjars/layui/css/layui.css",
                          AUri.CAPTCHA,
                          AUri.USER_ROUTER,
                         AUri.WEBSOCKET_SRV)
                .permitAll()
                // 除上面外的所有请求全部需要鉴权认证
                .anyRequest().authenticated()
                .and()
                .headers().frameOptions().disable();

        imageCodeFilter.setAuthenticationFailureHandler(authenticationFailureHandler);
        // 添加JWT filter 用户名登录
        httpSecurity
                // 添加图形证码校验过滤器
//                .addFilterBefore(imageCodeFilter, UsernamePasswordAuthenticationFilter.class)
                // 添加JWT验证过滤器
                .addFilterBefore(jwtAuthenticationTokenFilter, UsernamePasswordAuthenticationFilter.class)
                // 添加JWT验证过滤器
                .addFilterBefore(mySecurityPrivilegeFilter, FilterSecurityInterceptor.class);
    }

    @Autowired
    public void configureGlobal(AuthenticationManagerBuilder auth) throws Exception {
        auth.userDetailsService(userDetailsService) // 设置UserDetailsService
                .passwordEncoder(new BCryptPasswordEncoder()); // 使用BCrypt进行密码的hash
    }
}
